/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package gui;

import data.*;
import database.DatabaseInterface;
import hanson.HansonParser;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.table.DefaultTableModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import midi.MIDIAnalyser;
import midi.MIDIPlayer;

/**
 *
 * @author smuRf
 */
public class MainGui extends javax.swing.JFrame {
    private IScore iScore = null;
    private File midiFile;
    private String path;
    private HashMap<MotifType, ArrayList<IMotif>> detectedMotifs;
    private ColorRowRenderer rowRendererOriginal;
    private ColorRowRenderer rowRendererNormalized;
    public static String DB_USER;// = "PBMC_user";
    public static String DB_PASSWORD;// = "PBMC_pass";
    public static String DB_URL;// = "thesmurf.dyndns.org";
    private boolean loggedIn = false;
    private boolean saveLogin = false;
    private String username = null;
    private String password = null;
    private HashMap<Integer, Boolean> composeMotifs;
    private JTree activeMotifTree;
    private MotifType activeMotifType;
    private JTable activeDataTable;
    private ColorRowRenderer activeRowRenderer;
    
    /**
     * Creates new form MainGui
     */
    
    public MainGui() {
        path = "";
        
        initComponents();

        rowRendererOriginal = new ColorRowRenderer(taDataOriginal.getColumnCount(), new Color(57, 105, 138), Color.white, new Color(212, 212, 212), Color.white);
        rowRendererNormalized = new ColorRowRenderer(taDataNormalized.getColumnCount(), new Color(57, 105, 138), Color.white, new Color(212, 212, 212), Color.white);

        for(int i = 0; i < taDataOriginal.getColumnCount(); i++)
            taDataOriginal.getColumnModel().getColumn(i).setCellRenderer(rowRendererOriginal.getColumnRenderer(i));
        for(int i = 0; i < taDataNormalized.getColumnCount(); i++)
            taDataNormalized.getColumnModel().getColumn(i).setCellRenderer(rowRendererNormalized.getColumnRenderer(i));

        activeRowRenderer = rowRendererOriginal;

        detectedMotifs = new HashMap();
            
        Properties prop = null;
        try {
            prop = new Properties();
            prop.load(new FileInputStream("app.properties"));
        } catch (IOException ex) {
        }
        
        if(prop != null)
        {
            String motifs = prop.getProperty("SELECTED_MOTIFS");
            if(motifs != null)
            {
                if(!motifs.isEmpty())
                {
                    String[] selectedIds = motifs.split(";");
                    composeMotifs = new HashMap();
                    for(String id:selectedIds)
                        composeMotifs.put(Integer.parseInt(id), true);
                }
            }

            path = prop.getProperty("PATH");

            username = prop.getProperty("USERNAME");
            password = prop.getProperty("PASSWORD");
            
            DB_USER = prop.getProperty("DB_USER");
            DB_PASSWORD = prop.getProperty("DB_PASS");
            DB_URL = prop.getProperty("DB_URL");
        }

        if(username != null && password != null && DB_USER != null && DB_PASSWORD != null && DB_URL != null)
        {
            DatabaseInterface databaseInterface = new DatabaseInterface(DB_USER, DB_PASSWORD, DB_URL);
            if(databaseInterface.isDatabaseConnectionUp())
            {
                if(databaseInterface.login(username, password.getBytes()))
                {
                    loginStatus.setText("logged in as '"+username+"'");
                    loggedIn = true;
                    saveLogin = true;
                    login.setEnabled(false);
                    logout.setEnabled(true);
                }
                else
                {
                    loginStatus.setText("not logged in");
                    login.setEnabled(true);
                    logout.setEnabled(false);
                }
                databaseInterface.closeConnection();
            }
        }
        else
        {
            loginStatus.setText("not logged in");
            login.setEnabled(true);
            logout.setEnabled(false);
        }
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane2 = new javax.swing.JScrollPane();
        jTable1 = new javax.swing.JTable();
        toolbar = new javax.swing.JToolBar();
        btnPlay = new javax.swing.JButton();
        btnStop = new javax.swing.JButton();
        jSeparator5 = new javax.swing.JToolBar.Separator();
        btnDetect = new javax.swing.JButton();
        jSeparator6 = new javax.swing.JToolBar.Separator();
        btnDatabase = new javax.swing.JButton();
        btnDatabaseAdd = new javax.swing.JButton();
        jSeparator7 = new javax.swing.JToolBar.Separator();
        btnCompose = new javax.swing.JButton();
        pNav = new javax.swing.JPanel();
        spNav = new javax.swing.JScrollPane();
        tNav = new javax.swing.JTree();
        tabMotifs = new javax.swing.JTabbedPane();
        pnlRepetition = new javax.swing.JPanel();
        spMotifsRepetition = new javax.swing.JScrollPane();
        tMotifsRepetition = new javax.swing.JTree();
        pnlVariations = new javax.swing.JPanel();
        spMotifsVariations = new javax.swing.JScrollPane();
        tMotifsVariations = new javax.swing.JTree();
        pnlRhythmic = new javax.swing.JPanel();
        spMotifsRhythmic = new javax.swing.JScrollPane();
        tMotifsRhythmic = new javax.swing.JTree();
        spProps = new javax.swing.JScrollPane();
        taProps = new javax.swing.JTable();
        pData = new javax.swing.JPanel();
        tabData = new javax.swing.JTabbedPane();
        spDataOriginal = new javax.swing.JScrollPane();
        taDataOriginal = new javax.swing.JTable();
        spDataNormalized = new javax.swing.JScrollPane();
        taDataNormalized = new javax.swing.JTable();
        pStatus = new javax.swing.JPanel();
        loginStatus = new javax.swing.JLabel();
        jMenuBar = new javax.swing.JMenuBar();
        mFile = new javax.swing.JMenu();
        open = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JPopupMenu.Separator();
        play = new javax.swing.JMenuItem();
        stop = new javax.swing.JMenuItem();
        mMotif = new javax.swing.JMenu();
        detect = new javax.swing.JMenuItem();
        jSeparator3 = new javax.swing.JPopupMenu.Separator();
        add = new javax.swing.JMenuItem();
        mDatabase = new javax.swing.JMenu();
        db = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JPopupMenu.Separator();
        login = new javax.swing.JMenuItem();
        logout = new javax.swing.JMenuItem();
        jSeparator4 = new javax.swing.JPopupMenu.Separator();
        settings = new javax.swing.JMenuItem();
        mCompose = new javax.swing.JMenu();
        compose = new javax.swing.JMenuItem();

        jTable1.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane2.setViewportView(jTable1);

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Pattern-Based MIDI Composer");
        setIconImage(Toolkit.getDefaultToolkit()
            .getImage(getClass().getResource("/icons/noteIcon.png")));
        setPreferredSize(new java.awt.Dimension(1024, 768));
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });
        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                formComponentResized(evt);
            }
        });

        toolbar.setRollover(true);

        btnPlay.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/play.png"))); // NOI18N
        btnPlay.setToolTipText("Play");
        btnPlay.setFocusable(false);
        btnPlay.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        btnPlay.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        btnPlay.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnPlayActionPerformed(evt);
            }
        });
        toolbar.add(btnPlay);

        btnStop.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/stop.png"))); // NOI18N
        btnStop.setToolTipText("Stop");
        btnStop.setFocusable(false);
        btnStop.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        btnStop.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        btnStop.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnStopActionPerformed(evt);
            }
        });
        toolbar.add(btnStop);
        toolbar.add(jSeparator5);

        btnDetect.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/detect.png"))); // NOI18N
        btnDetect.setToolTipText("Detect motifs");
        btnDetect.setFocusable(false);
        btnDetect.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        btnDetect.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        btnDetect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnDetectActionPerformed(evt);
            }
        });
        toolbar.add(btnDetect);
        toolbar.add(jSeparator6);

        btnDatabase.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/database.png"))); // NOI18N
        btnDatabase.setToolTipText("Motif database");
        btnDatabase.setFocusable(false);
        btnDatabase.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        btnDatabase.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        btnDatabase.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnDatabaseActionPerformed(evt);
            }
        });
        toolbar.add(btnDatabase);

        btnDatabaseAdd.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/database_add.png"))); // NOI18N
        btnDatabaseAdd.setToolTipText("Add to database");
        btnDatabaseAdd.setFocusable(false);
        btnDatabaseAdd.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        btnDatabaseAdd.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        btnDatabaseAdd.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnDatabaseAddActionPerformed(evt);
            }
        });
        toolbar.add(btnDatabaseAdd);
        toolbar.add(jSeparator7);

        btnCompose.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/irc-voice.png"))); // NOI18N
        btnCompose.setFocusable(false);
        btnCompose.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        btnCompose.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        btnCompose.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnComposeActionPerformed(evt);
            }
        });
        toolbar.add(btnCompose);

        getContentPane().add(toolbar, java.awt.BorderLayout.PAGE_START);

        pNav.setPreferredSize(new java.awt.Dimension(230, 10));
        pNav.setLayout(new java.awt.BorderLayout());

        spNav.setPreferredSize(new java.awt.Dimension(51, 256));

        javax.swing.tree.DefaultMutableTreeNode treeNode1 = new javax.swing.tree.DefaultMutableTreeNode("score");
        tNav.setModel(new javax.swing.tree.DefaultTreeModel(treeNode1));
        tNav.setEnabled(false);
        tNav.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                tNavValueChanged(evt);
            }
        });
        tNav.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                tNavFocusGained(evt);
            }
            public void focusLost(java.awt.event.FocusEvent evt) {
                tNavFocusLost(evt);
            }
        });
        spNav.setViewportView(tNav);

        pNav.add(spNav, java.awt.BorderLayout.PAGE_START);

        tabMotifs.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                tabMotifsStateChanged(evt);
            }
        });

        pnlRepetition.setLayout(new java.awt.BorderLayout());

        spMotifsRepetition.setPreferredSize(new java.awt.Dimension(74, 256));

        treeNode1 = new javax.swing.tree.DefaultMutableTreeNode("motifs");
        tMotifsRepetition.setModel(new javax.swing.tree.DefaultTreeModel(treeNode1));
        tMotifsRepetition.setEnabled(false);
        tMotifsRepetition.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                tMotifsRepetitionValueChanged(evt);
            }
        });
        tMotifsRepetition.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                tMotifsRepetitionFocusGained(evt);
            }
            public void focusLost(java.awt.event.FocusEvent evt) {
                tMotifsRepetitionFocusLost(evt);
            }
        });
        spMotifsRepetition.setViewportView(tMotifsRepetition);

        pnlRepetition.add(spMotifsRepetition, java.awt.BorderLayout.CENTER);

        tabMotifs.addTab("Repetitions", pnlRepetition);

        pnlVariations.setLayout(new java.awt.BorderLayout());

        spMotifsVariations.setPreferredSize(new java.awt.Dimension(74, 256));

        treeNode1 = new javax.swing.tree.DefaultMutableTreeNode("motifs");
        tMotifsVariations.setModel(new javax.swing.tree.DefaultTreeModel(treeNode1));
        tMotifsVariations.setEnabled(false);
        tMotifsVariations.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                tMotifsVariationsValueChanged(evt);
            }
        });
        tMotifsVariations.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                tMotifsVariationsFocusGained(evt);
            }
            public void focusLost(java.awt.event.FocusEvent evt) {
                tMotifsVariationsFocusLost(evt);
            }
        });
        spMotifsVariations.setViewportView(tMotifsVariations);

        pnlVariations.add(spMotifsVariations, java.awt.BorderLayout.CENTER);

        tabMotifs.addTab("Variations", pnlVariations);

        pnlRhythmic.setLayout(new java.awt.BorderLayout());

        treeNode1 = new javax.swing.tree.DefaultMutableTreeNode("motifs");
        tMotifsRhythmic.setModel(new javax.swing.tree.DefaultTreeModel(treeNode1));
        tMotifsRhythmic.setEnabled(false);
        tMotifsRhythmic.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                tMotifsRhythmicValueChanged(evt);
            }
        });
        tMotifsRhythmic.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                tMotifsRhythmicFocusGained(evt);
            }
            public void focusLost(java.awt.event.FocusEvent evt) {
                tMotifsRhythmicFocusLost(evt);
            }
        });
        spMotifsRhythmic.setViewportView(tMotifsRhythmic);

        pnlRhythmic.add(spMotifsRhythmic, java.awt.BorderLayout.CENTER);

        tabMotifs.addTab("Rhytmic", pnlRhythmic);

        pNav.add(tabMotifs, java.awt.BorderLayout.CENTER);

        spProps.setPreferredSize(new java.awt.Dimension(452, 256));

        taProps.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "property", "value"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.Object.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        taProps.setEnabled(false);
        spProps.setViewportView(taProps);

        pNav.add(spProps, java.awt.BorderLayout.PAGE_END);

        getContentPane().add(pNav, java.awt.BorderLayout.LINE_START);

        pData.setLayout(new javax.swing.BoxLayout(pData, javax.swing.BoxLayout.LINE_AXIS));

        tabData.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                tabDataStateChanged(evt);
            }
        });

        taDataOriginal.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "index", "post hanson", "pre hanson", "post diff", "pre diff", "note", "octave", "duration", "pitch"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.Integer.class, java.lang.String.class, java.lang.String.class, java.lang.Integer.class, java.lang.Integer.class, java.lang.String.class, java.lang.Integer.class, java.lang.Double.class, java.lang.Integer.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false, false, false, false, false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        taDataOriginal.setEnabled(false);
        taDataOriginal.getTableHeader().setReorderingAllowed(false);
        spDataOriginal.setViewportView(taDataOriginal);

        tabData.addTab("original", spDataOriginal);

        taDataNormalized.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "index", "post hanson", "pre hanson", "post diff", "pre diff", "note", "octave", "duration", "pitch"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.Integer.class, java.lang.String.class, java.lang.String.class, java.lang.Integer.class, java.lang.Integer.class, java.lang.String.class, java.lang.Integer.class, java.lang.Double.class, java.lang.Integer.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false, false, false, false, false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        taDataNormalized.setEnabled(false);
        taDataNormalized.getTableHeader().setReorderingAllowed(false);
        spDataNormalized.setViewportView(taDataNormalized);

        tabData.addTab("normalized", spDataNormalized);

        pData.add(tabData);

        getContentPane().add(pData, java.awt.BorderLayout.CENTER);

        pStatus.setPreferredSize(new java.awt.Dimension(10, 20));
        pStatus.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT, 5, 0));
        pStatus.add(loginStatus);

        getContentPane().add(pStatus, java.awt.BorderLayout.PAGE_END);

        mFile.setText("File");

        open.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.CTRL_MASK));
        open.setText("open");
        open.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                openActionPerformed(evt);
            }
        });
        mFile.add(open);
        mFile.add(jSeparator1);

        play.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_P, java.awt.event.InputEvent.CTRL_MASK));
        play.setText("play");
        play.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                playActionPerformed(evt);
            }
        });
        mFile.add(play);

        stop.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.CTRL_MASK));
        stop.setText("stop");
        stop.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopActionPerformed(evt);
            }
        });
        mFile.add(stop);

        jMenuBar.add(mFile);

        mMotif.setText("Motif");

        detect.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_D, java.awt.event.InputEvent.CTRL_MASK));
        detect.setText("detect");
        detect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                detectActionPerformed(evt);
            }
        });
        mMotif.add(detect);
        mMotif.add(jSeparator3);

        add.setText("add to database");
        add.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addActionPerformed(evt);
            }
        });
        mMotif.add(add);

        jMenuBar.add(mMotif);

        mDatabase.setText("Database");

        db.setText("motif database");
        db.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                dbActionPerformed(evt);
            }
        });
        mDatabase.add(db);
        mDatabase.add(jSeparator2);

        login.setText("login");
        login.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loginActionPerformed(evt);
            }
        });
        mDatabase.add(login);

        logout.setText("logout");
        logout.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                logoutActionPerformed(evt);
            }
        });
        mDatabase.add(logout);
        mDatabase.add(jSeparator4);

        settings.setText("change password");
        settings.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                settingsActionPerformed(evt);
            }
        });
        mDatabase.add(settings);

        jMenuBar.add(mDatabase);

        mCompose.setText("Compose");

        compose.setText("compose");
        compose.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                composeActionPerformed(evt);
            }
        });
        mCompose.add(compose);

        jMenuBar.add(mCompose);

        setJMenuBar(jMenuBar);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void openActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openActionPerformed
        midiFile = null;
        JFileChooser fc = new JFileChooser(path);
        taDataOriginal.requestFocus();
        int returnVal = fc.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            clearNavTreeView();
            clearDataTableView();
            clearPropsTableView();
            updateMotifsTreeView(null, MotifType.REPETITION);
            updateMotifsTreeView(null, MotifType.VARIATION);
            updateMotifsTreeView(null, MotifType.RHYTHMIC);
            
            tNav.setEnabled(true);
            tNav.requestFocus();

            tNav.clearSelection();
            tMotifsRepetition.clearSelection();
            tMotifsVariations.clearSelection();
            tMotifsRhythmic.clearSelection();
            taDataOriginal.clearSelection();
            taDataNormalized.clearSelection();
            
            if(detectedMotifs != null)
                detectedMotifs.clear(); 
            detectedMotifs = null;
           
            midiFile = fc.getSelectedFile();
            
            path = midiFile.getParentFile().getAbsolutePath();
            
            MIDIAnalyser midiAnalyser = new MIDIAnalyser(midiFile);
            iScore = midiAnalyser.analyse();
            
            updateNavTreeView();
            
            tNav.expandRow(0);
            rowRendererOriginal.clearEquiRows();
            rowRendererOriginal.clearBaseRows();
            rowRendererNormalized.clearEquiRows();
            rowRendererNormalized.clearBaseRows();
        }
    }//GEN-LAST:event_openActionPerformed

    public void setLoginData(boolean successful, String user, String pass, boolean save)
    {
        if(successful)
        {
            loggedIn = true;
            username = user;
            password = pass;
            loginStatus.setText("logged in as '"+username+"'");
            login.setEnabled(false);
            logout.setEnabled(true);
        }
        else
        {
            loggedIn = false;
            loginStatus.setText("not logged in");
            login.setEnabled(true);
            logout.setEnabled(false);
        }
        
        saveLogin = save;
    }
    
    public String getLoggedInUsername()
    {
        return username;
    }
    
    private void clearDataTableView()
    {
        DefaultTableModel tableModel = (DefaultTableModel)taDataOriginal.getModel();
        int rowCount = tableModel.getRowCount();
        
        for(int i = 0; i < rowCount; i++)
            tableModel.removeRow(0);
        
        tableModel = (DefaultTableModel)taDataNormalized.getModel();
        rowCount = tableModel.getRowCount();
        
        for(int i = 0; i < rowCount; i++)
            tableModel.removeRow(0);
    }
    
    private void updateDataTableView(boolean showOriginal)
    {
        clearDataTableView();
        
        DefaultTableModel tableModel = (DefaultTableModel)activeDataTable.getModel();
        
        int[] ids = TreeSelector.getSelectedId(tNav.getSelectionPath());
        
        if(ids != null)
        {
            if(ids.length == 2)
            {
                IPhrase phrase = null;
                if(showOriginal)
                    phrase = iScore.getIPart(ids[1]).getOriginalIPhrase(ids[0]);
                else
                    phrase = iScore.getIPart(ids[1]).getNormalizedIPhrase(ids[0]);
                    
                ArrayList<INote> iNotes = phrase.getINotes();
                int lastPitch = iNotes.get(0).getIPitch();
                for(int i = 0 ; i < iNotes.size(); i++)
                {
                    String strPostHanson = "";
                    double postHansonValue = phrase.getPostHansonValue(i);
                    if(postHansonValue != -3)
                        strPostHanson = HansonParser.getStringForValue(postHansonValue);
                    
                    String strPreHanson = "";
                    double preHansonValue = phrase.getPreHansonValue(i);
                    if(preHansonValue != -3)
                        strPreHanson = HansonParser.getStringForValue(preHansonValue);
                    
                    INote iNote = iNotes.get(i);
                    int postPitchDiff = iNote.getIPitch() - lastPitch;
                    iNote.setPostPitchDiff(postPitchDiff);
                    
                    int prePitchDiff = 0;
                    if(i < (iNotes.size()-1))
                    {
                        INote nextNote = iNotes.get(i+1);
                        prePitchDiff = nextNote.getIPitch() - iNote.getIPitch();
                        iNote.setPrePitchDiff(prePitchDiff);
                    }
                    
                    tableModel.insertRow(tableModel.getRowCount(), new Object[]{i, strPostHanson, strPreHanson, postPitchDiff, prePitchDiff, iNote.getINote(), iNote.getIOctave(), iNote.getRhythmValue(), iNote.getIPitch()});
                    lastPitch = iNote.getIPitch();
                }
            }
        }
    }
    
    private void clearPropsTableView()
    {
        DefaultTableModel tableModel = (DefaultTableModel)taProps.getModel();
        int rowCount = tableModel.getRowCount();
        
        for(int i = 0; i < rowCount; i++)
            tableModel.removeRow(0);
    }
    
    private void updatePropsTableViewFromNavTree()
    {
        clearPropsTableView();
        
        DefaultTableModel tableModel = (DefaultTableModel)taProps.getModel();        
        
        int[] ids = TreeSelector.getSelectedId(tNav.getSelectionPath());
        
        if(ids != null && iScore != null)
        {
            if(ids.length == 0)
            {
                tableModel.insertRow(tableModel.getRowCount(), new Object[]{"name", iScore.getName()});
                tableModel.insertRow(tableModel.getRowCount(), new Object[]{"tempo", iScore.getTempo()});
                tableModel.insertRow(tableModel.getRowCount(), new Object[]{"timing", iScore.getNumerator() + "/" + iScore.getDenominator()});
                tableModel.insertRow(tableModel.getRowCount(), new Object[]{"ppqn", iScore.getPPQN()});
            } 
            if(ids.length == 1)
            {
                IPart iPart = iScore.getIPart(ids[0]);
                
                tableModel.insertRow(tableModel.getRowCount(), new Object[]{"instrument", iPart.getDefaultInstrument()});
                tableModel.insertRow(tableModel.getRowCount(), new Object[]{"channel", iPart.getChannel()});
            }
            if(ids.length == 2)
            {
                IPhrase iPhrase;
                if(activeDataTable == taDataOriginal)
                    iPhrase = iScore.getIPart(ids[1]).getOriginalIPhrase(ids[0]);
                else
                    iPhrase = iScore.getIPart(ids[1]).getNormalizedIPhrase(ids[0]);
                
                tableModel.insertRow(tableModel.getRowCount(), new Object[]{"start time", iPhrase.getStartTime()});
            }
        }
    }
    
    private void updatePropsTableViewFromMotifTree()
    {
        DefaultTableModel tableModel = (DefaultTableModel)taProps.getModel();
        int rowCount = tableModel.getRowCount();
        
        for(int i = 0; i < rowCount; i++)
            tableModel.removeRow(0);
        
        int[] ids = TreeSelector.getSelectedId(activeMotifTree.getSelectionPath());
        
        if(ids != null && detectedMotifs != null)
        {
            if(ids.length == 0)
                tableModel.insertRow(tableModel.getRowCount(), new Object[]{"#motifs", detectedMotifs.get(activeMotifType).size()});
            if(ids.length == 1)
            {
                IScore motif = detectedMotifs.get(activeMotifType).get(ids[0]).getIScore();
                
                tableModel.insertRow(tableModel.getRowCount(), new Object[]{"name", TreeSelector.getSelectedName(activeMotifTree.getSelectionPath())});
                tableModel.insertRow(tableModel.getRowCount(), new Object[]{"tempo", motif.getTempo()});
                tableModel.insertRow(tableModel.getRowCount(), new Object[]{"#motif matches", detectedMotifs.get(activeMotifType).get(ids[0]).getEquiMotifCount()});
            }
            if(ids.length == 2)
            {
                IMotif selectedMotif = detectedMotifs.get(activeMotifType).get(ids[1]).getEquiMotif(ids[0]);
                tableModel.insertRow(tableModel.getRowCount(), new Object[]{"name", TreeSelector.getSelectedName(activeMotifTree.getSelectionPath())});
                tableModel.insertRow(tableModel.getRowCount(), new Object[]{"type", selectedMotif.getMotifType()});
            }
        }
    }
    
    private void clearNavTreeView()
    {
        DefaultMutableTreeNode root = (DefaultMutableTreeNode)tNav.getModel().getRoot();
        root.removeAllChildren();
    }
    
    private void updateNavTreeView()
    {   
        clearNavTreeView();
        DefaultMutableTreeNode root = (DefaultMutableTreeNode)tNav.getModel().getRoot();
        
        for(IPart iPart:iScore.getIParts())
        {
            DefaultMutableTreeNode part = new DefaultMutableTreeNode(iPart.getName());
            
            for(IPhrase iPhrase:iPart.getOriginalIPhrases())
            {
                DefaultMutableTreeNode phrase = new DefaultMutableTreeNode(iPhrase.getName());
                part.add(phrase);
            }
            root.add(part);
        }
        tNav.updateUI();
    }
    
    private void updateMotifsTreeView(ArrayList<IMotif> motifs, MotifType type)
    {   
        JTree motifTree;
        if(type == MotifType.REPETITION)
            motifTree = tMotifsRepetition;
        else if(type == MotifType.VARIATION)
            motifTree = tMotifsVariations;
        else if(type == MotifType.RHYTHMIC)
            motifTree = tMotifsRhythmic;
        else
            motifTree = activeMotifTree;
        
        DefaultMutableTreeNode root = (DefaultMutableTreeNode)motifTree.getModel().getRoot();
        root.removeAllChildren();
        
        if(motifs != null)
        {
            for(IMotif motif:motifs)
            {
                IScore motifScore = motif.getIScore();
                DefaultMutableTreeNode motifNode = new DefaultMutableTreeNode(motifScore.getName());
                
                for(IMotif match:motif.getEquiMotifs())
                {
                    DefaultMutableTreeNode matchNode = new DefaultMutableTreeNode(match.getIScore().getName());
                    motifNode.add(matchNode);
                }
                root.add(motifNode);
            }
        }
        else
        {
            motifTree.setEnabled(false);
        }
        motifTree.updateUI();
    }
    
    private void play()
    {
        if(activeMotifTree.hasFocus())
        {
            int[] ids = TreeSelector.getSelectedId(activeMotifTree.getSelectionPath());
            
            if(ids != null)
            {
                IMotif motif = null;
                
                if(ids.length == 1)
                    motif = detectedMotifs.get(activeMotifType).get(ids[0]);
                if(ids.length == 2)
                    motif = detectedMotifs.get(activeMotifType).get(ids[1]).getEquiMotif(ids[0]);
                
                if(ids.length > 0)
                    MIDIPlayer.play(motif);
            }
        }
        else
        {
            if(iScore != null)
            {
                if(tNav.getSelectionPath() == null)
                    tNav.setSelectionRow(0);
                
                int[] ids = TreeSelector.getSelectedId(tNav.getSelectionPath());
                
                if(ids.length == 0)
                    MIDIPlayer.play(iScore);
                if(ids.length == 1)
                    MIDIPlayer.play(iScore.getIPart(ids[0]));
                if(ids.length == 2)
                {
                    if(activeDataTable == taDataOriginal)
                        MIDIPlayer.play(iScore.getIPart(ids[1]).getOriginalIPhrase(ids[0]));
                    else
                        MIDIPlayer.play(iScore.getIPart(ids[1]).getNormalizedIPhrase(ids[0]));
                }
            }
        }
    }
    
    private void stop()
    {
        MIDIPlayer.stop();
    }
    
    private void playActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_playActionPerformed
        play();
    }//GEN-LAST:event_playActionPerformed

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        Properties prop = new Properties();
        
        if(midiFile!=null)
        {
            prop.setProperty("PATH", midiFile.getParentFile().getAbsolutePath());
        }
        else
        {
            prop.setProperty("PATH", path==null?"":path);
        }
        
        if(saveLogin)
        {
            prop.setProperty("USERNAME", username);
            prop.setProperty("PASSWORD", password);
        }
        
        String selectedMotifs = "";
        if(composeMotifs != null)
        {
            for(int id:composeMotifs.keySet())
                selectedMotifs += id+";";
        }
        
        prop.setProperty("SELECTED_MOTIFS", selectedMotifs);
        prop.setProperty("DB_USER", DB_USER == null ? "" : DB_USER);
        prop.setProperty("DB_PASS", DB_PASSWORD == null ? "" : DB_PASSWORD);
        prop.setProperty("DB_URL", DB_URL == null ? "" : DB_URL);
            
        try {
            prop.store(new FileOutputStream("app.properties"), null);
        } catch (IOException ex) {
            Logger.getLogger(MainGui.class.getName()).log(Level.SEVERE, null, ex);
        }
    }//GEN-LAST:event_formWindowClosing

    private void stopActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopActionPerformed
        stop();
    }//GEN-LAST:event_stopActionPerformed

    private void tNavValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_tNavValueChanged
        if(activeDataTable == taDataOriginal)
            updateDataTableView(true);
        else
            updateDataTableView(false);
        updatePropsTableViewFromNavTree();
        
        activeRowRenderer.clearEquiRows();
        activeRowRenderer.clearBaseRows();
        if(activeMotifTree.getSelectionCount() > 0)
        {
            int[] navIds = TreeSelector.getSelectedId(tNav.getSelectionPath());
            int[] motifIds = TreeSelector.getSelectedId(activeMotifTree.getAnchorSelectionPath());
            
            if(navIds != null && detectedMotifs != null && navIds.length == 2)
            {
                int phraseId = navIds[0];
                int partId = navIds[1];
                int motifId;
                
                if(motifIds.length > 0)
                    motifId = motifIds[0];
                else
                    return;
                
                ArrayList<IMotif> iMotifs = detectedMotifs.get(activeMotifType);
                
                if(iMotifs == null)
                    return;
                
                IMotif iMotif = iMotifs.get(motifId);
                if(iMotif.getSrcPartId() == partId && iMotif.getSrcPhraseId() == phraseId)
                {
                    ListSelectionModel selectionModel = (ListSelectionModel)activeDataTable.getSelectionModel();
                    tNav.setSelectionPath(getNavTreePathFromIDs(iMotif.getSrcPartId(), iMotif.getSrcPhraseId()));
                    
                    if(motifIds.length == 1)
                    {
                        int startBaseIndex = iMotif.getSrcNoteId();
                    int endBaseIndex = startBaseIndex + iMotif.getMotifLength() - 1;

                    int[] baseRows = new int[endBaseIndex - startBaseIndex + 1];
                    for(int index = 0; index <= (endBaseIndex - startBaseIndex); index++)
                        baseRows[index] = index + startBaseIndex;

                    activeRowRenderer.addBaseRows(baseRows);
                    activeRowRenderer.clearEquiRows();
                        
                        for(IMotif equiMotif:iMotif.getEquiMotifs())
                        {
                            int startIndex = equiMotif.getSrcNoteId();
                            int endIndex = startIndex + equiMotif.getMotifLength() - 1;

                            int[] rows = new int[endIndex - startIndex + 1];
                            for(int index = 0; index <= (endIndex - startIndex); index++)
                                rows[index] = index + startIndex;

                            activeRowRenderer.addEquiRows(rows);
                        }
                    }
                    if(motifIds.length == 2)
                    {
                        activeRowRenderer.clearEquiRows();
                        activeRowRenderer.clearBaseRows();

                        IMotif match = detectedMotifs.get(activeMotifType).get(motifIds[1]).getEquiMotif(motifIds[0]);
                        
                        int startIndex = match.getSrcNoteId();
                            int endIndex = startIndex + match.getMotifLength() - 1;

                            int[] rows = new int[endIndex - startIndex + 1];
                            for(int index = 0; index <= (endIndex - startIndex); index++)
                                rows[index] = index + startIndex;

                        activeRowRenderer.addBaseRows(rows);
                    }
                }
            }
        }
    }//GEN-LAST:event_tNavValueChanged

    private void btnPlayActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnPlayActionPerformed
        play();
    }//GEN-LAST:event_btnPlayActionPerformed

    private void btnStopActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnStopActionPerformed
        stop();
    }//GEN-LAST:event_btnStopActionPerformed

    public void setDetectedMotifs(HashMap<MotifType, ArrayList<IMotif>> motifs)
    {
        tMotifsRepetition.clearSelection();
        tMotifsVariations.clearSelection();
        tMotifsRhythmic.clearSelection();
        activeDataTable.clearSelection();
        rowRendererOriginal.clearEquiRows();
        rowRendererOriginal.clearBaseRows();
        rowRendererNormalized.clearEquiRows();
        rowRendererNormalized.clearBaseRows();
        
        detectedMotifs = motifs;
        
        for(MotifType motifType:detectedMotifs.keySet())
        {
            if(!detectedMotifs.get(motifType).isEmpty())
            {
                switch(motifType)
                {
                    case REPETITION:
                        tMotifsRepetition.setEnabled(true);
                        updateMotifsTreeView(detectedMotifs.get(motifType), MotifType.REPETITION);
                        break;
                    case VARIATION:
                        tMotifsVariations.setEnabled(true);
                        updateMotifsTreeView(detectedMotifs.get(motifType), MotifType.VARIATION);
                        break;
                    case RHYTHMIC:
                        tMotifsRhythmic.setEnabled(true);
                        updateMotifsTreeView(detectedMotifs.get(motifType), MotifType.RHYTHMIC);
                        break;
                }
            }
            else
            {
                DefaultMutableTreeNode root = null;
                switch(motifType)
                {
                    case REPETITION:
                        root = (DefaultMutableTreeNode)tMotifsRepetition.getModel().getRoot();
                        root.removeAllChildren();
                        tMotifsRepetition.setEnabled(false);
                        tMotifsRepetition.updateUI();
                        break;
                    case VARIATION:
                        root = (DefaultMutableTreeNode)tMotifsVariations.getModel().getRoot();
                        root.removeAllChildren();
                        tMotifsVariations.setEnabled(false);
                        tMotifsVariations.updateUI();
                        break;
                    case RHYTHMIC:
                        root = (DefaultMutableTreeNode)tMotifsRhythmic.getModel().getRoot();
                        root.removeAllChildren();
                        tMotifsRhythmic.setEnabled(false);
                        tMotifsRhythmic.updateUI();
                        break;
                }
            }
        }
    }
    
    private void detectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_detectActionPerformed
        btnDetectActionPerformed(null);
        /*int[] ids = TreeSelector.getSelectedId(tNav.getSelectionPath());
        
        if(ids != null && ids.length == 2)
        {
            MotifDetection mDetection = new MotifDetection(3, 7);
            
            int phraseId = ids[0];
            int partId = ids[1];
            
            detectedMotifs = mDetection.detect(iScore.getIPart(partId).getIPhrase(phraseId));
            
            for(MotifType motifType:detectedMotifs.keySet())
            {
                if(!detectedMotifs.get(motifType).isEmpty())
                {
                    switch(motifType)
                    {
                        case REPETITION:
                            tMotifsRepetition.setEnabled(true);
                            updateMotifsTreeView(detectedMotifs.get(motifType), MotifType.REPETITION);
                            break;
                        case VARIATION:
                            tMotifsVariations.setEnabled(true);
                            updateMotifsTreeView(detectedMotifs.get(motifType), MotifType.VARIATION);
                            break;
                        case RHYTHMIC:
                            tMotifsRhythmic.setEnabled(true);
                            updateMotifsTreeView(detectedMotifs.get(motifType), MotifType.RHYTHMIC);
                            break;
                    }
                }
                else
                {
                    DefaultMutableTreeNode root = null;
                    switch(motifType)
                    {
                        case REPETITION:
                            root = (DefaultMutableTreeNode)tMotifsRepetition.getModel().getRoot();
                            root.removeAllChildren();
                            tMotifsRepetition.setEnabled(false);
                            tMotifsRepetition.updateUI();
                            break;
                        case VARIATION:
                            root = (DefaultMutableTreeNode)tMotifsVariations.getModel().getRoot();
                            root.removeAllChildren();
                            tMotifsVariations.setEnabled(false);
                            tMotifsVariations.updateUI();
                            break;
                        case RHYTHMIC:
                            root = (DefaultMutableTreeNode)tMotifsRhythmic.getModel().getRoot();
                            root.removeAllChildren();
                            tMotifsRhythmic.setEnabled(false);
                            tMotifsRhythmic.updateUI();
                            break;
                    }
                }
            }
        }
        else
            JOptionPane.showMessageDialog(this, "You have to select a phrase", "Error", JOptionPane.ERROR_MESSAGE);*/
    }//GEN-LAST:event_detectActionPerformed

    private void formComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_formComponentResized
        int navPanelHeight = (int) (pNav.getSize().getHeight());
        
        spMotifsRepetition.setSize(new Dimension(74, navPanelHeight/5*2));
        spNav.setPreferredSize(new Dimension(74, navPanelHeight/5*2));
        spProps.setPreferredSize(new Dimension(74, navPanelHeight/5)); 
        pNav.updateUI();
    }//GEN-LAST:event_formComponentResized

    private TreePath getNavTreePathFromIDs(int partId, int phraseId)
    {
        TreeNode[] phrasePath = null;
        DefaultMutableTreeNode partNode = null;
        
        DefaultMutableTreeNode root = (DefaultMutableTreeNode)tNav.getModel().getRoot();
        
        Enumeration partEnum = root.children();
        while(partEnum.hasMoreElements())
        {
            partNode = (DefaultMutableTreeNode)partEnum.nextElement();
            String partName = (String)partNode.getUserObject();
            
            if(partName.equals("part"+partId))
                break;
        }
        
        Enumeration phraseEnum = partNode.children();
        while(phraseEnum.hasMoreElements())
        {
            DefaultMutableTreeNode phraseNode = (DefaultMutableTreeNode)phraseEnum.nextElement();
            String phraseName = (String)phraseNode.getUserObject();
            
            if(phraseName.equals("phrase"+phraseId))
            {
                phrasePath = phraseNode.getPath();
                break;
            }
        }
        
        return new TreePath(phrasePath);
    }
    
    private void motifTreeValueChanged()
    {
        updatePropsTableViewFromMotifTree();
        
        int[] ids = TreeSelector.getSelectedId(activeMotifTree.getSelectionPath());
        tabData.setSelectedIndex(1);
        if(ids != null && activeDataTable == taDataNormalized)
        {
            ListSelectionModel selectionModel = (ListSelectionModel)activeDataTable.getSelectionModel();
            
            if(ids.length == 0)
            {
                activeRowRenderer.clearEquiRows();
                activeRowRenderer.clearBaseRows();
                selectionModel.clearSelection();
            }
            if(ids.length == 1)
            {
                activeRowRenderer.clearBaseRows();
                IMotif iMotif = detectedMotifs.get(activeMotifType).get(ids[0]);
                
                int startBaseIndex = iMotif.getSrcNoteId();
                int endBaseIndex = startBaseIndex + iMotif.getMotifLength() - 1;

                int[] baseRows = new int[endBaseIndex - startBaseIndex + 1];
                for(int index = 0; index <= (endBaseIndex - startBaseIndex); index++)
                    baseRows[index] = index + startBaseIndex;

                activeRowRenderer.addBaseRows(baseRows);
                activeRowRenderer.clearEquiRows();
                
                for(IMotif equiMotif:iMotif.getEquiMotifs())
                {
                    tNav.setSelectionPath(getNavTreePathFromIDs(equiMotif.getSrcPartId(), equiMotif.getSrcPhraseId()));

                    int startIndex = equiMotif.getSrcNoteId();
                    int endIndex = startIndex + equiMotif.getMotifLength() - 1;
                    
                    int[] rows = new int[endIndex - startIndex + 1];
                    for(int index = 0; index <= (endIndex - startIndex); index++)
                        rows[index] = index + startIndex;
                        
                    activeRowRenderer.addEquiRows(rows);
                }
                updatePropsTableViewFromMotifTree();
            }
            if(ids.length == 2)
            {
                activeRowRenderer.clearEquiRows();
                activeRowRenderer.clearBaseRows();
                
                IMotif match = detectedMotifs.get(activeMotifType).get(ids[1]).getEquiMotif(ids[0]);
                    
                tNav.setSelectionPath(getNavTreePathFromIDs(match.getSrcPartId(), match.getSrcPhraseId()));
                updatePropsTableViewFromMotifTree();

                int startIndex = match.getSrcNoteId();
                int endIndex = startIndex + match.getMotifLength() - 1;

                int[] rows = new int[endIndex - startIndex + 1];
                for(int index = 0; index <= (endIndex - startIndex); index++)
                    rows[index] = index + startIndex;

                activeRowRenderer.addBaseRows(rows);
            }
            
            activeDataTable.updateUI();
        }
    }
    
    private void tMotifsRepetitionValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_tMotifsRepetitionValueChanged
        motifTreeValueChanged();
    }//GEN-LAST:event_tMotifsRepetitionValueChanged

    private void tMotifsRepetitionFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_tMotifsRepetitionFocusGained
        tMotifsRepetition.setBorder(new LineBorder(new Color(115, 164, 209)));
        updatePropsTableViewFromMotifTree();
    }//GEN-LAST:event_tMotifsRepetitionFocusGained

    private void tMotifsRepetitionFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_tMotifsRepetitionFocusLost
        tMotifsRepetition.setBorder(null);
    }//GEN-LAST:event_tMotifsRepetitionFocusLost

    private void tNavFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_tNavFocusGained
        tNav.setBorder(new LineBorder(new Color(115, 164, 209)));
        updatePropsTableViewFromNavTree();
    }//GEN-LAST:event_tNavFocusGained

    private void tNavFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_tNavFocusLost
        tNav.setBorder(null);
    }//GEN-LAST:event_tNavFocusLost
        
    public void setComposeMotifs(HashMap composeMotifs)
    {
        this.composeMotifs = composeMotifs;
    }
    
    private void dbActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_dbActionPerformed
        MotifDatabaseDialog motifDetectionDialog = new MotifDatabaseDialog(this, true, composeMotifs);
        
        if(loggedIn)
        {
            motifDetectionDialog.setVisible(true);
        }
        else
        {
            DatabaseLoginDialog loginDialog = new DatabaseLoginDialog(this, true, motifDetectionDialog); 
            loginDialog.setVisible(true);
        }
    }//GEN-LAST:event_dbActionPerformed

    private void settingsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_settingsActionPerformed
        DatabaseSettingsDialog databaseSettingsDialog = new DatabaseSettingsDialog(this, true);
        
        if(loggedIn)
        {
            databaseSettingsDialog.setVisible(true);
        }
        else
        {
            DatabaseLoginDialog loginDialog = new DatabaseLoginDialog(this, true, databaseSettingsDialog); 
            loginDialog.setVisible(true);
        }
    }//GEN-LAST:event_settingsActionPerformed

    private void logoutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_logoutActionPerformed
        int ret = JOptionPane.showConfirmDialog(this, "Do you really want to logout?", "Logout", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
        if(ret == JOptionPane.YES_OPTION)
            setLoginData(false, "", "", false);
    }//GEN-LAST:event_logoutActionPerformed

    private void loginActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loginActionPerformed
        DatabaseLoginDialog loginDialog = new DatabaseLoginDialog(this, true); 
        loginDialog.setVisible(true);
    }//GEN-LAST:event_loginActionPerformed

    private void btnDatabaseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnDatabaseActionPerformed
        dbActionPerformed(null);
    }//GEN-LAST:event_btnDatabaseActionPerformed

    private void addActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addActionPerformed
        if(!detectedMotifs.isEmpty())
        {
            int idList[][] = TreeSelector.getSelectedId(activeMotifTree.getSelectionPaths());
            
            if(idList != null && idList[0].length > 0)
            {
                JCheckBox includeEquiMotifs = new JCheckBox("Include motif matches");
                String msg = "Do you want to add the selected motifs to the database?\n"; 
                Object[] msgContent;
                if(idList[0].length == 1)
                    msgContent = new Object[]{msg, includeEquiMotifs};
                else
                    msgContent = new Object[]{msg};
                
                int ret = JOptionPane.showConfirmDialog(this, msgContent, "Add to database", JOptionPane.YES_NO_OPTION); 
                if(ret == JOptionPane.YES_OPTION)
                {
                    for(int ids[]:idList)
                    {
                        DatabaseInterface databaseInterface = new DatabaseInterface(DB_USER, DB_PASSWORD, DB_URL);

                        if(!loggedIn)
                        {
                            DatabaseLoginDialog loginDialog = new DatabaseLoginDialog(this, true); 
                            loginDialog.setVisible(true);
                        }
                            
                        if(loggedIn)
                        {
                            if(ids.length == 1)
                            {
                                databaseInterface.insertMotif(detectedMotifs.get(activeMotifType).get(ids[0]), username, 0);

                                if(includeEquiMotifs.isSelected())
                                {
                                    for(IMotif equiMotif:detectedMotifs.get(activeMotifType).get(ids[0]).getEquiMotifs())
                                    {
                                        databaseInterface.insertMotif(equiMotif, username, 0);
                                    }
                                }
                            }
                            if(ids.length == 2)
                                databaseInterface.insertMotif(detectedMotifs.get(activeMotifType).get(ids[1]).getEquiMotif(ids[0]), username, 0);
                        }

                        databaseInterface.closeConnection();
                    }
                }
            }
            else
                JOptionPane.showMessageDialog(this, "You have to select a motif", "Error", JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_addActionPerformed

    private void btnDatabaseAddActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnDatabaseAddActionPerformed
        if(!detectedMotifs.isEmpty())
        {
            int idList[][] = TreeSelector.getSelectedId(activeMotifTree.getSelectionPaths());
            
            if(idList != null && idList[0].length > 0)
            {
                MotifAddDialog addDialog = new MotifAddDialog(this, true, idList, activeMotifType, username, detectedMotifs);
                addDialog.setLocationRelativeTo(this);

                if(loggedIn)
                {
                    addDialog.setVisible(true);
                }
                else
                {
                    DatabaseLoginDialog loginDialog = new DatabaseLoginDialog(this, true, addDialog); 
                    loginDialog.setVisible(true);
                }
            }
        }
        //addActionPerformed(null);
    }//GEN-LAST:event_btnDatabaseAddActionPerformed

    private void btnDetectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnDetectActionPerformed
        DetectionDialog detectionDialog = new DetectionDialog(this, true);
        int[] ids = TreeSelector.getSelectedId(tNav.getSelectionPath());
        
        if(ids != null && ids.length == 2)
        {
            int phraseId = ids[0];
            int partId = ids[1];
            tabData.setSelectedIndex(1);
            if(activeDataTable == taDataOriginal)
                detectionDialog.setIPhrase(iScore.getIPart(partId).getOriginalIPhrase(phraseId));
            else
                detectionDialog.setIPhrase(iScore.getIPart(partId).getNormalizedIPhrase(phraseId));
        }
        else
        {
            JOptionPane.showMessageDialog(this, "You have to select a phrase", "Error", JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        if(loggedIn)
        {
            detectionDialog.setVisible(true);
        }
        else
        {
            DatabaseLoginDialog loginDialog = new DatabaseLoginDialog(this, true, detectionDialog); 
            loginDialog.setVisible(true);
        }
    }//GEN-LAST:event_btnDetectActionPerformed

    private void btnComposeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnComposeActionPerformed
        if(composeMotifs != null && !composeMotifs.isEmpty())
        {
            ComposingDialog composingDialog = new ComposingDialog(this, true, composeMotifs);

            if(loggedIn)
            {
                composingDialog.setVisible(true);
            }
            else
            {
                DatabaseLoginDialog loginDialog = new DatabaseLoginDialog(this, true, composingDialog); 
                loginDialog.setVisible(true);
            }
        }
        else
           JOptionPane.showMessageDialog(this, "You have to select at least one motif for composing", "Error", JOptionPane.ERROR_MESSAGE); 
    }//GEN-LAST:event_btnComposeActionPerformed

    private void composeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_composeActionPerformed
        btnComposeActionPerformed(evt);
    }//GEN-LAST:event_composeActionPerformed

    private void tMotifsVariationsValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_tMotifsVariationsValueChanged
        motifTreeValueChanged();
    }//GEN-LAST:event_tMotifsVariationsValueChanged

    private void tMotifsVariationsFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_tMotifsVariationsFocusGained
        tMotifsVariations.setBorder(new LineBorder(new Color(115, 164, 209)));
        updatePropsTableViewFromMotifTree();
    }//GEN-LAST:event_tMotifsVariationsFocusGained

    private void tMotifsVariationsFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_tMotifsVariationsFocusLost
        tMotifsVariations.setBorder(null);
    }//GEN-LAST:event_tMotifsVariationsFocusLost

    private void tabMotifsStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_tabMotifsStateChanged
        if(tabMotifs.getSelectedIndex() == MotifType.REPETITION.ordinal())
        {
            activeMotifTree = tMotifsRepetition;
            activeMotifType = MotifType.REPETITION;
        }
        if(tabMotifs.getSelectedIndex() == MotifType.VARIATION.ordinal())
        {
            activeMotifTree = tMotifsVariations;
            activeMotifType = MotifType.VARIATION;
        }
        if(tabMotifs.getSelectedIndex() == MotifType.RHYTHMIC.ordinal())
        {
            activeMotifTree = tMotifsRhythmic;
            activeMotifType = MotifType.RHYTHMIC;
        }
    }//GEN-LAST:event_tabMotifsStateChanged

    private void tMotifsRhythmicFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_tMotifsRhythmicFocusGained
        tMotifsRhythmic.setBorder(new LineBorder(new Color(115, 164, 209)));
        updatePropsTableViewFromMotifTree();
    }//GEN-LAST:event_tMotifsRhythmicFocusGained

    private void tMotifsRhythmicFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_tMotifsRhythmicFocusLost
        tMotifsRhythmic.setBorder(null);
    }//GEN-LAST:event_tMotifsRhythmicFocusLost

    private void tMotifsRhythmicValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_tMotifsRhythmicValueChanged
        motifTreeValueChanged();
    }//GEN-LAST:event_tMotifsRhythmicValueChanged

    private void tabDataStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_tabDataStateChanged
        if(tabData.getSelectedIndex() == 0)
        {
            activeDataTable = taDataOriginal;
            activeRowRenderer = rowRendererOriginal;
            updateDataTableView(true);
        }
        if(tabData.getSelectedIndex() == 1)
        {
            activeDataTable = taDataNormalized;
            activeRowRenderer = rowRendererNormalized;
            updateDataTableView(false);
        }
        if(activeRowRenderer != null)
        {
            activeRowRenderer.clearBaseRows();
            activeRowRenderer.clearEquiRows();
        }
        if(tabData.getSelectedIndex() == 1)
            motifTreeValueChanged();
    }//GEN-LAST:event_tabDataStateChanged

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        /*
         * Set the Nimbus look and feel
         */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /*
         * If Nimbus (introduced in Java SE 6) is not available, stay with the
         * default look and feel. For details see
         * http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException | InstantiationException | IllegalAccessException | javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(MainGui.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>

        /*
         * Create and display the form
         */
        java.awt.EventQueue.invokeLater(new Runnable() {

            @Override
            public void run() {
                new MainGui().setVisible(true);
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem add;
    private javax.swing.JButton btnCompose;
    private javax.swing.JButton btnDatabase;
    private javax.swing.JButton btnDatabaseAdd;
    private javax.swing.JButton btnDetect;
    private javax.swing.JButton btnPlay;
    private javax.swing.JButton btnStop;
    private javax.swing.JMenuItem compose;
    private javax.swing.JMenuItem db;
    private javax.swing.JMenuItem detect;
    private javax.swing.JMenuBar jMenuBar;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JPopupMenu.Separator jSeparator1;
    private javax.swing.JPopupMenu.Separator jSeparator2;
    private javax.swing.JPopupMenu.Separator jSeparator3;
    private javax.swing.JPopupMenu.Separator jSeparator4;
    private javax.swing.JToolBar.Separator jSeparator5;
    private javax.swing.JToolBar.Separator jSeparator6;
    private javax.swing.JToolBar.Separator jSeparator7;
    private javax.swing.JTable jTable1;
    private javax.swing.JMenuItem login;
    private javax.swing.JLabel loginStatus;
    private javax.swing.JMenuItem logout;
    private javax.swing.JMenu mCompose;
    private javax.swing.JMenu mDatabase;
    private javax.swing.JMenu mFile;
    private javax.swing.JMenu mMotif;
    private javax.swing.JMenuItem open;
    private javax.swing.JPanel pData;
    private javax.swing.JPanel pNav;
    private javax.swing.JPanel pStatus;
    private javax.swing.JMenuItem play;
    private javax.swing.JPanel pnlRepetition;
    private javax.swing.JPanel pnlRhythmic;
    private javax.swing.JPanel pnlVariations;
    private javax.swing.JMenuItem settings;
    private javax.swing.JScrollPane spDataNormalized;
    private javax.swing.JScrollPane spDataOriginal;
    private javax.swing.JScrollPane spMotifsRepetition;
    private javax.swing.JScrollPane spMotifsRhythmic;
    private javax.swing.JScrollPane spMotifsVariations;
    private javax.swing.JScrollPane spNav;
    private javax.swing.JScrollPane spProps;
    private javax.swing.JMenuItem stop;
    private javax.swing.JTree tMotifsRepetition;
    private javax.swing.JTree tMotifsRhythmic;
    private javax.swing.JTree tMotifsVariations;
    private javax.swing.JTree tNav;
    private javax.swing.JTable taDataNormalized;
    private javax.swing.JTable taDataOriginal;
    private javax.swing.JTable taProps;
    private javax.swing.JTabbedPane tabData;
    private javax.swing.JTabbedPane tabMotifs;
    private javax.swing.JToolBar toolbar;
    // End of variables declaration//GEN-END:variables
}

